/****************************************************
AboutSQLServer.com blog
Written by Dmitri Korotkevitch

CLR: Security considerations
2013-08-13
*****************************************************/
set nocount on
go

sp_configure 'show advanced options', 1;
reconfigure;
GO
sp_configure 'clr enabled', 1;
reconfigure;
GO
sp_configure 'show advanced options', 0;
reconfigure;
go

use master
go

if exists(select * from sys.databases where name = 'CLRSecurity')
	raiserror('Database CLRSecurity already exists',21,0) with log
go

create database CLRSecurity
go

use CLRSecurity
go

 -- Failed
create assembly [CLRSecurity]
authorization [dbo]
from 'c:\SQL\CLRSecurity\CLRSecurity.dll'
with permission_set = external_access;
go

use master
go

create master key encryption 
by password = 'Pas$word1'
go

create asymmetric key KeyExternalAccess
from file = 'c:\SQL\CLRSecurity\CLRSecurityKey.snk'
go

create login CLRExtAccessLogin 
from asymmetric key KeyExternalAccess
go

grant external access assembly to CLRExtAccessLogin
go

use CLRSecurity
go

create assembly [CLRSecurity]
authorization [dbo]
from 'c:\SQL\CLRSecurity\CLRSecurity.dll'
with permission_set = external_access;
go

create procedure [dbo].[Dummy]
as external name 
	[CLRSecurity].[StoredProcedures].[Dummy]
go

create function [dbo].[GetOrderTotalCLR] (@orderId int)
returns money
as external name 
	[CLRSecurity].[UserDefinedFunctions].[GetOrderTotalCLR]
go

exec dbo.Dummy
go


-- Ownership chaining
create table dbo.Orders
(
	OrderId int not null,

	constraint PK_Orders
	primary key clustered(OrderId)
)
go

create table dbo.OrderLineItems
(
	OrderId int not null,
	OrderLineItemId int not null,
	ArticleId int not null,
	Quantity decimal(9,3) not null,
	Price money not null,

	constraint PK_OrderLineItems
	primary key clustered(OrderId, OrderLineItemId)
)
go

insert into dbo.Orders(OrderId) values(1)

insert into dbo.OrderLineItems
(OrderId,OrderLineItemId,ArticleId, Quantity, Price)
values(1,1,1,3,9.99), (1,2,2,2,4.99)
go

create function dbo.GetOrderTotal(@OrderId int)
returns money
as
begin
	return 
	(
		select sum(Quantity * Price) as Total 
		from dbo.OrderLineItems
		where OrderId = @OrderId
	)
end
go

create view dbo.vOrdersTSQL(OrderId, OrderTotal)
as
	select o.OrderId, dbo.GetOrderTotal(o.OrderId)
	from dbo.Orders o
go

create view dbo.vOrdersCLR(OrderId, OrderTotal)
as
	select o.OrderId, dbo.GetOrderTotalCLR(o.OrderId)
	from dbo.Orders o
go

create user Bob without login;
grant select on object::dbo.vOrdersTSQL to [Bob];
grant select on object::dbo.vOrdersCLR to [Bob];
go

execute as user='Bob'
go

-- Success
select * from dbo.vOrdersTSQL

-- Failure
select * from dbo.vOrdersCLR
go

revert
go


grant select on object::dbo.OrderLineItems to [Bob];
go

execute as user='Bob'
go

-- Success
select * from dbo.vOrdersCLR
go

revert
go


-- Cleanup
use master
go

alter database CLRSecurity set single_user with rollback immediate
go

drop database CLRSecurity
go

drop login CLRExtAccessLogin
go

drop asymmetric key KeyExternalAccess
go

